#!/bin/bash 
# sysgrep - jwbensley@gmail.com
# 
# This script will watch a given log file for new lines added every five minutes
# and grep out these new lines, matching them against grep patterns in the "matches" 
# file and emailing them to the below address

function replaceIPs () 
{
echo "$1" > result.tmp # Store the syslog line passed
# Extract a list of IPs from the log line
grep -Eo '([0-9]{1,3}\.){3}[0-9]{1,3}'  < result.tmp > ip.tmp 

# Loop through and replace IPs with their host name
lastip=""
while read IP
do
    if [ "$IP" != "$lastip" ]
    then
        host=`host $IP | awk '{print $NF}'`
	echo -e "$host\n"
        if [ "$host" == "3(NXDOMAIN)" ] # Check if the syslog has a reverse PTR record
        then
            echo "$host" > result2.tmp
         else 
            sed '0,/'${IP}'/s/'${IP}'/'{$host}'/' < result.tmp > result2.tmp
            mv result2.tmp result.tmp
            lastip=$IP
        fi
    fi
done < ip.tmp
rm ip.tmp 
}

# Email address to alert to
email="alerts@noc.network.net"

# Where is this script installed
scriptdir="/opt/sysgrep"
logfiletmp="$scriptdir/logfile.tmp"
cleanlogtmp="$scriptdir/cleanlog.tmp"

# Syslog file (like /var/log/network.log)
logfile=$1
if [ ! -f $logfile ]
then
 echo "Log file $1 does not exist"
 exit 1
fi

# File with list of grep strings, one per line
matches="$scriptdir/matches"
if [ ! -f $matches ]
then
 echo "Grep patterns file $matches does not exist"
 exit 1
fi

notmatches="$scriptdir/notmatches"
if [ ! -f $notmatches ]
then
 echo "Grep exclude patterns file $notmatches does not exist"
 exit 1
fi

# Time in seconds between each check of the log file
delay=300


##########################################
####### NO MORE CONFIG BELOW HERE ########
##########################################

# Current line count of log file
# (start from $2 is given)
lastC=`grep -c "" $logfile`
if [ $# -eq 2 ]
then
 lastC=$2
fi

# Start the main loop
while true;
do

 # Get the current number of lines in the syslog file
 thisC=`grep -c "" $logfile`
 diffC=0
 # Has anything new been logged that we need to check?
 if [ $thisC -gt $lastC ]
 then
  # Yes, so get the number of new lines and update our previous line count
  diffC=`expr $thisC - $lastC`
  lastC=$thisC # Update for next time
 fi
 # Check for less lines (in the case of logrotate for example)
 if [ $thisC -lt $lastC ]
 then
  diffC=$thisC # If less lines we will start again and grep the whole thing so far
  lastC=$thisC
 fi

 # Ignore comments in the matches files, and blank lines
 grep -v ^# $matches | grep -v ^$ > $matches.tmp
 grep -v ^# $notmatches | grep -v ^$ > $notmatches.tmp
 # Extract new log lines
 tail -n $diffC $logfile > $logfiletmp
 
 count=$diffC
 # Grep out each of the NOT patterns from the new lines of the log file
 while [ $count -gt 0 ]
 do
  safe=""
  # Load up the NOT patterns
  while IFS=$'\n' read -r -a string
  do
   result=$(tail -n $count $logfiletmp | head -n 1 | grep -v "${string[0]}")
   if [ "$result" == "" ]
   then
    safe=${string[0]}
   fi
  done < $notmatches.tmp
  if [ "$safe" != "" ]
  then
   grep -v $safe $logfiletmp >> $cleanlogtmp
   mv $cleanlogtmp $logfiletmp
  fi
  let "count=$count-1"
 done

 count=$diffC
 # Grep out each of the new lines of the log file with the match patterns
 while [ $count -gt 0 ]
 do
  # Load up the grep patterns
  while IFS=$'\n' read -r -a string
  do
   result=$(tail -n $count $logfiletmp | head -n 1 | grep -m 1 "${string[0]}")
   if [ "$result" != "" ] # We have a valid syslog line to send
   then
    # Replace IPs with hostnames
    replaceIPs "$result"
    result=`cat result.tmp`
    rm result.tmp
    echo "$result" | mail -s "Syslog" "$email"
    break 
   fi
  done < $matches.tmp
  let "count=$count-1"
 done

 # We are always deleting the re-greping the matches file so that new entries
 # can be added to it without having to re-start the script
 rm $matches.tmp
 rm $notmatches.tmp
 rm $logfiletmp
 sleep $delay

done
